(function() {
  var Messenger, RequestMiddleware, crema, dolce, errors;

  dolce = require("dolce");

  RequestMiddleware = require("./middleware");

  crema = require("crema");

  Messenger = require("./messenger");

  errors = require("../errors");

  /*
  
  Director process:
  */

  module.exports = (function() {
    /*
    	 some directors are passive, meaning errors aren't returned if a route does not exist. This goes for collectors,
    	 emitters, etc.
    */
    _Class.prototype.passive = false;

    /*
    	 constructor
    */

    function _Class(name, router) {
      this.name = name;
      this.router = router;
      this._collection = dolce.collection();
    }

    /*
    	 returns number of listeners based on path given
    */

    _Class.prototype.numListeners = function(path, ops) {
      return this._collection.get(path, ops).chains.length;
    };

    /*
    	 dispatches a request
    */

    _Class.prototype.dispatch = function(requestWriter) {
      var chain, chains, messanger, middleware, numChains, numRunning, oldAck, requestReader, _i, _len;
      try {
        chains = this.getListeners(requestWriter, void 0, !this.passive);
      } catch (e) {
        return requestWriter.callback(new Error("" + this.name + " " + e.message));
      }
      numChains = chains.length;
      numRunning = numChains;
      oldAck = requestWriter.callback;
      requestWriter.running = !!numChains;
      requestWriter.callback = function() {
        requestWriter.running = !!(--numRunning);
        if (oldAck) {
          return oldAck.apply(this, Array.apply(null, arguments).concat([numRunning, numChains]));
        }
      };
      if (!!!chains.length && !this.passive) {
        requestWriter.callback(new errors.NotFoundError("" + this.name + " route \"" + (crema.stringifySegments(requestWriter.path.segments)) + "\" does not exist"));
        return this;
      }
      for (_i = 0, _len = chains.length; _i < _len; _i++) {
        chain = chains[_i];
        requestReader = requestWriter.reader();
        middleware = RequestMiddleware.wrap(chain, requestWriter.pre, requestWriter.next, this);
        messanger = this._newMessenger(requestReader, middleware);
        messanger.start();
      }
      return this;
    };

    /*
    	 adds a route listener to the collection tree
    */

    _Class.prototype.addListener = function(route, callback) {
      disposable;
      var disposable, oldCallback;
      if (route.tags.one) {
        oldCallback = callback;
        callback = function() {
          oldCallback.apply(this, arguments);
          return disposable.dispose();
        };
      }
      this._validateListener(route, callback);
      return disposable = this._collection.add(route, callback);
    };

    /*
    */

    _Class.prototype.removeListeners = function(route) {
      return this._collection.remove(route.path, {
        tags: route.tags
      });
    };

    /*
    */

    _Class.prototype.paths = function(ops) {
      var listener, _i, _len, _ref, _results;
      _ref = this._collection.find(ops);
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        listener = _ref[_i];
        _results.push({
          tags: listener.tags,
          type: this.name,
          value: listener.path,
          segments: listener.segments
        });
      }
      return _results;
    };

    /*
    */

    _Class.prototype.listenerQuery = function(ops) {
      var filter, key, tag;
      filter = [];
      for (key in ops.filter) {
        tag = {};
        tag[key] = ops.filter[key];
        filter.push(tag);
      }
      return {
        $or: [
          {
            $and: filter
          }, {
            unfilterable: {
              $exists: true
            }
          }
        ]
      };
    };

    /*
    */

    _Class.prototype.getListeners = function(request, expand, throwError) {
      return this._collection.get(request.path, {
        siftTags: this.listenerQuery(request),
        expand: expand,
        throwErrors: throwError
      }).chains;
    };

    /*
    	 returns a new request
    */

    _Class.prototype._newMessenger = function(request, middleware) {
      return new Messenger(request, middleware, this);
    };

    /*
    */

    _Class.prototype._validateListener = function(route) {
      var listeners;
      if (this.passive) return;
      listeners = this._collection.get(route.path, {
        tags: route.tags,
        expand: false
      });
      if (!!listeners.length) {
        throw new Error("Route \"" + route.path.value + "\" already exists");
      }
    };

    return _Class;

  })();

}).call(this);
