(function() {
  var Reader, Stream, disposable,
    __hasProp = Object.prototype.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; };

  Stream = require("stream").Stream;

  disposable = require("disposable");

  module.exports = Reader = (function(_super) {

    __extends(Reader, _super);

    /*
    */

    function Reader(source) {
      this.source = source;
      Reader.__super__.constructor.call(this);
      this.setMaxListeners(0);
      this._listen();
    }

    /*
    	 needs to be overridable incase there's more stuff to listen to (headers)
    */

    Reader.prototype._listenTo = function() {
      return ["data", "end", "error"];
    };

    /*
    */

    Reader.prototype._listen = function() {
      var event, listeners, _fn, _i, _len, _ref,
        _this = this;
      this._buffer = [];
      listeners = disposable.create();
      if (this.source) {
        _ref = this._listenTo();
        _fn = function(event) {
          var onEvent;
          onEvent = function(arg1, arg2) {
            _this._started = true;
            return _this.emit(event, arg1, arg2);
          };
          _this.source.on(event, onEvent);
          return listeners.add(function() {
            return _this.source.removeListener(event, onEvent);
          });
        };
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          event = _ref[_i];
          _fn(event);
        }
      }
      this.on("data", function(data, encoding) {
        if (!_this._cache) return;
        return _this._buffer.push({
          chunk: data,
          encoding: encoding
        });
      });
      this.on("end", function() {
        if (_this.ended) throw new Error("Cannot end more than once");
        return _this.ended = true;
      });
      return this.on("error", function(err) {
        return _this.error = err;
      });
    };

    /*
    */

    Reader.prototype.setEncoding = function(encoding) {
      var _ref;
      return (_ref = this.source) != null ? _ref.setEncoding(encoding) : void 0;
    };

    /*
    */

    Reader.prototype.pause = function() {
      var _ref;
      return (_ref = this.source) != null ? typeof _ref.pause === "function" ? _ref.pause() : void 0 : void 0;
    };

    /*
    */

    Reader.prototype.resume = function() {
      var _ref;
      return (_ref = this.source) != null ? typeof _ref.resume === "function" ? _ref.resume() : void 0 : void 0;
    };

    /*
    */

    Reader.prototype.destroy = function() {
      var _ref;
      return (_ref = this.source) != null ? typeof _ref.destroy === "function" ? _ref.destroy() : void 0 : void 0;
    };

    /*
    */

    Reader.prototype.destroySoon = function() {
      var _ref;
      return (_ref = this.source) != null ? typeof _ref.destroySoon === "function" ? _ref.destroySoon() : void 0 : void 0;
    };

    /*
    	 flags the reader that data should be cached as it's coming in.
    */

    Reader.prototype.cache = function(value) {
      if (arguments.length) this._cache = value || !!this._buffer.length;
      return this._cache;
    };

    /*
     	 listens on a reader, and pipes it to a callback a few ways
    */

    Reader.prototype.dump = function(callback, ops) {
      var pipedStream, wrappedCallback;
      if (!ops) ops = {};
      wrappedCallback = this._dumpCallback(callback, ops);
      pipedStream = this._started ? new Reader(this) : this;
      wrappedCallback.call(this, null, pipedStream);
      if (!this._started) return;
      return this._dumpCached(pipedStream, ops);
    };

    /*
    */

    Reader.prototype._dumpCallback = function(callback, ops) {
      var listeners, pipeTo,
        _this = this;
      if (callback instanceof Stream) {
        ops.stream = true;
        pipeTo = callback;
        callback = function(err, stream) {
          var type, _fn, _i, _len, _ref;
          _ref = _this._listenTo();
          _fn = function(type) {
            return stream.on(type, function() {
              return pipeTo.emit.apply(pipeTo, [type].concat(Array.prototype.slice.call(arguments)));
            });
          };
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            type = _ref[_i];
            _fn(type);
          }
          return null;
        };
      }
      if (typeof callback === 'object') {
        ops.stream = true;
        listeners = callback;
        callback = function(err, stream) {
          var type, _results;
          _results = [];
          for (type in listeners) {
            _results.push(stream.on(type, listeners[type]));
          }
          return _results;
        };
      }
      if (ops.stream) return callback;
      return function(err, reader) {
        var buffer, onEnd;
        if (err) return callback(err);
        buffer = [];
        onEnd = function(err) {
          var chunk, _i, _len, _results;
          if (ops.batch) return callback.call(_this, err, buffer);
          if (!buffer.length) return callback.call(_this, err);
          if (ops.each) {
            _results = [];
            for (_i = 0, _len = buffer.length; _i < _len; _i++) {
              chunk = buffer[_i];
              _results.push(callback.call(_this, err, chunk));
            }
            return _results;
          } else {
            return callback.call(_this, err, buffer.length > 1 ? buffer : buffer[0]);
          }
        };
        reader.on("data", function(data, encoding) {
          return buffer.push(data);
        });
        reader.on("error", onEnd);
        return reader.on("end", onEnd);
      };
    };

    /*
    */

    Reader.prototype._dumpCached = function(pipedReader) {
      var data, _i, _len, _ref;
      _ref = this._buffer;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        data = _ref[_i];
        pipedReader.emit("data", data.chunk, data.encoding);
      }
      if (this.ended) pipedReader.emit("end");
      if (this.error) return pipedReader.emit("error");
    };

    return Reader;

  })(Stream);

  Reader.prototype.readable = true;

}).call(this);
